@@ -13,13 +13,13 @@ import time
 from collections.abc import AsyncGenerator, Callable, Sequence
 from copy import copy
 from dataclasses import dataclass, field
-from typing import Any
+from typing import Unpack
 
 from flexai.capability import Capability
 from flexai.llm import Client, DefaultClient
+from flexai.llm.client import PartialAgentRunArgs
 from flexai.message import (
     AIMessage,
-    DataBlock,
     Message,
     MessageContent,
     SystemMessage,
@@ -33,11 +33,9 @@ from flexai.message import (
 from flexai.tool import Tool, send_message
 
 
-@dataclass(frozen=True, kw_only=True)
-class AgentConfig:
-    """Configuration options for the agent."""
-
-    extra_llm_params: dict[str, Any] = field(default_factory=dict)
+@dataclass(kw_only=True)
+class AgentEnv:
+    """The Agent's environment. Passed around to components that want access to the outside world."""
 
 
 @dataclass(frozen=True, kw_only=True)
@@ -64,18 +62,27 @@ class Agent:
     capabilities: list[Capability] = field(default_factory=list)
 
     # The language model to use for the agent.
-    llm: Client = field(default_factory=DefaultClient)
+    llms: Sequence[Client] = field(default_factory=lambda: [DefaultClient()])
 
-    # Additional configuration options for the agent.
-    config: AgentConfig = AgentConfig()
+    # The context that the agent is working in.
+    agent_env: AgentEnv = field(default_factory=AgentEnv)
 
     # The mapping of tool names to tool functions.
     toolbox: dict[str, Tool] = field(default_factory=dict, init=False)
 
     def __post_init__(self):
-        """Perform post-initialization setup."""
+        """Perform post-initialization setup.
+
+        Raises:
+            ValueError: If no clients were provided.
+        """
         tools = self.tools or []
 
+        if not self.llms:
+            raise ValueError(
+                "Agent must be instantiated with at least one LLM client.",
+            )
+
         # Convert callables to tools and store them in the toolbox.
         tools = {Tool.from_function(tool) for tool in set(tools)}
         tools = {tool.name: tool for tool in tools}
@@ -144,9 +151,32 @@ class Agent:
                 elif isinstance(output, SystemMessage):
                     system = output
 
-        # Cache the system message.
+        # Return the system message.
         yield system
 
+    async def get_tools(
+        self,
+    ) -> AsyncGenerator[MessageContent | Sequence[Tool], None]:
+        """Hook to modify the tool list before sending it to the LLM.
+
+        Yields:
+            Intermediate message chunks followed by the modified tool list.
+        """
+        tools = list(self.toolbox.values())
+
+        for capability in self.capabilities:
+            async for output in capability.modify_tools(tools):
+                # This is a partial message chunk.
+                if isinstance(output, MessageContent):
+                    yield output
+
+                # This is the modified tool list.
+                else:
+                    tools = output
+
+        # Return the tools.
+        yield tools
+
     async def modify_response(
         self, messages: list[Message], response: AIMessage
     ) -> AsyncGenerator[MessageContent | AIMessage, None]:
@@ -199,7 +229,19 @@ class Agent:
             Intermediate message chunks followed by the final tool result.
 
         """
-        tool = self.toolbox[tool_call.name]
+        try:
+            tool = self.toolbox[tool_call.name]
+        except KeyError:
+            yield ToolResult(
+                tool_call_id=tool_call.id,
+                result=TextBlock(
+                    text=f"The tool `{tool_call.name}` is not available to you."
+                ),
+                execution_time=0,
+                is_error=True,
+            )
+            return
+
         is_error = False
 
         # Add the context to the tool input.
@@ -209,7 +251,7 @@ class Agent:
             kwargs["messages"] = messages
 
             # Add the agent instance.
-            kwargs["agent"] = self
+            kwargs["agent_env"] = self.agent_env
 
         start_time = time.time()
         try:
@@ -241,11 +283,6 @@ class Agent:
 
         end_time = time.time()
 
-        if isinstance(result, dict):
-            result = DataBlock(data=result)
-        elif not isinstance(result, MessageContent):
-            result = TextBlock(text=str(result))
-
         yield ToolResult(
             tool_call_id=tool_call.id,
             result=result,
@@ -285,14 +322,144 @@ class Agent:
             ):
                 yield output
 
+    async def _get_chat_response(
+        self,
+        messages: list[Message],
+        system: str | SystemMessage,
+        tools: Sequence[Tool],
+        current_llm_index: int = 0,
+        llm_exception_callback: Callable[[Exception, int], bool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
+    ):
+        """Get a chat response from the LLM with automatic fallback between clients.
+
+        Attempts to get a response from the current LLM client. If an exception
+        occurs, optionally uses the callback to determine whether to retry with
+        the next client in the list.
+
+        Args:
+            messages: The conversation messages to send to the LLM.
+            system: The system message or prompt.
+            tools: The available tools for the LLM to use.
+            current_llm_index: Index of the current LLM client to use.
+            llm_exception_callback: Optional callback to determine retry behavior
+                when exceptions occur. Receives the exception and current LLM index,
+                returns True to retry with next client, False to raise the exception.
+            **input_args: Additional arguments to pass to the LLM client.
+
+        Returns:
+            The AI response message from the LLM.
+
+        Raises:
+            Exception: Any exception from the LLM client if no more retries available
+                or callback indicates not to retry.
+        """
+        try:
+            return await self.llms[current_llm_index].get_chat_response(
+                messages,
+                system=system,
+                tools=tools,
+                **input_args,
+            )
+
+        except Exception as e:
+            # Whether we should retry on this exception
+            should_retry = True
+
+            # If the caller provided a callback, obtain a recommendation for whether to retry
+            if llm_exception_callback is not None:
+                should_retry = llm_exception_callback(e, current_llm_index)
+
+            # If we have no more clients to try, raise
+            if current_llm_index == len(self.llms) - 1:
+                should_retry = False
+
+            if not should_retry:
+                raise
+
+            return await self._get_chat_response(
+                messages=messages,
+                system=system,
+                tools=tools,
+                current_llm_index=current_llm_index + 1,
+                llm_exception_callback=llm_exception_callback,
+                **input_args,
+            )
+
+    async def _stream_chat_response(
+        self,
+        messages: list[Message],
+        system: str | SystemMessage,
+        tools: Sequence[Tool],
+        current_llm_index: int = 0,
+        llm_exception_callback: Callable[[Exception, int], bool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
+    ):
+        """Stream a chat response from the LLM with automatic fallback between clients.
+
+        Attempts to stream a response from the current LLM client. If an exception
+        occurs, optionally uses the callback to determine whether to retry with
+        the next client in the list.
+
+        Args:
+            messages: The conversation messages to send to the LLM.
+            system: The system message or prompt.
+            tools: The available tools for the LLM to use.
+            current_llm_index: Index of the current LLM client to use.
+            llm_exception_callback: Optional callback to determine retry behavior
+                when exceptions occur. Receives the exception and current LLM index,
+                returns True to retry with next client, False to raise the exception.
+            **input_args: Additional arguments to pass to the LLM client.
+
+        Yields:
+            Streaming chunks of the AI response from the LLM.
+
+        Raises:
+            Exception: Any exception from the LLM client if no more retries available
+                or callback indicates not to retry.
+        """
+        try:
+            # Try streaming on this client.
+            async for output in self.llms[current_llm_index].stream_chat_response(
+                messages=messages,
+                system=system,
+                tools=tools,
+                **input_args,
+            ):
+                yield output
+
+        except Exception as e:
+            # Whether we should retry on this exception
+            should_retry = True
+
+            # If the caller provided a callback, obtain a recommendation for whether to retry
+            if llm_exception_callback is not None:
+                should_retry = llm_exception_callback(e, current_llm_index)
+
+            # If we have no more clients to try, raise
+            if current_llm_index == len(self.llms) - 1:
+                should_retry = False
+
+            if not should_retry:
+                raise
+
+            # Try the next LLM in our list of options
+            async for output in self._stream_chat_response(
+                messages=messages,
+                system=system,
+                tools=tools,
+                current_llm_index=current_llm_index + 1,
+                llm_exception_callback=llm_exception_callback,
+                **input_args,
+            ):
+                yield output
+
     async def step(
         self,
         messages: list[Message],
-        stream: bool = False,
-        allow_tool: bool = True,
-        force_tool: bool = True,
-        thinking_budget: int | None = None,
-        **kwargs,
+        stream: bool,
+        llm_exception_callback: Callable[[Exception, int], bool] | None = None,
+        **run_args: Unpack[PartialAgentRunArgs],
     ) -> AsyncGenerator[MessageContent | Message | list[Message], None]:
         """Process a single turn in the conversation.
 
@@ -301,11 +468,13 @@ class Agent:
 
         Args:
             messages: Current conversation messages.
-            stream: Boolean flag for whether to yield partial message chunks from the API or not.
-            allow_tool: Boolean flag for whether to fully interpret tool calls/results in the history.
-            force_tool: Boolean flag for whether to force the use of tools.
-            thinking_budget: Budget for thinking tokens.
-            **kwargs: Additional keyword arguments to pass to the client.
+            stream: Whether to stream the response.
+            llm_exception_callback: Optional callback to determine retry behavior
+                when LLM exceptions occur. Receives the exception and current LLM index,
+                returns True to retry with next client, False to raise the exception.
+            **run_args: Additional arguments including allow_tool, force_tool,
+                thinking_budget, model, temperature, raise_on_null_content and other
+                keyword arguments to pass to the client.
 
         Yields:
             Partial chunks of the tool response, if streaming is on.
@@ -338,16 +507,25 @@ class Agent:
             # This is the modified system message.
             else:
                 system = output
+
+        tools = []
+        async for output in self.get_tools():
+            # This is a partial message chunk
+            if isinstance(output, MessageContent):
+                yield output
+
+            # This is the modified tool list.
+            else:
+                tools = output
+
         if stream:
             response = None
-            async for chunk in self.llm.stream_chat_response(
+            async for chunk in self._stream_chat_response(
                 messages,
                 system=system,
-                tools=list(self.toolbox.values()),
-                allow_tool=allow_tool,
-                thinking_budget=thinking_budget,
-                force_tool=force_tool,
-                **kwargs,
+                tools=tools,
+                llm_exception_callback=llm_exception_callback,
+                **run_args,
             ):
                 # This is a partial message chunk.
                 if isinstance(chunk, MessageContent):
@@ -358,17 +536,16 @@ class Agent:
                     response = chunk
         else:
             # Get the response from the LLM.
-            response = await self.llm.get_chat_response(
+            response = await self._get_chat_response(
                 messages,
                 system=system,
-                tools=list(self.toolbox.values()),
-                force_tool=force_tool,
-                thinking_budget=thinking_budget,
-                **kwargs,
+                tools=tools,
+                llm_exception_callback=llm_exception_callback,
+                **run_args,
             )
 
         if response is None:
-            raise ValueError(f"No response received from the LLM {self.llm}.")
+            raise ValueError(f"No response received from the LLM {self.llms}.")
 
         modified_response = None
 
@@ -415,10 +592,8 @@ class Agent:
         self,
         messages: list[Message],
         stream: bool = False,
-        allow_tool: bool = True,
-        force_tool: bool = True,
-        thinking_budget: int | None = None,
-        **kwargs,
+        llm_exception_callback: Callable[[Exception, int], bool] | None = None,
+        **run_args: Unpack[PartialAgentRunArgs],
     ) -> AsyncGenerator[MessageContent | Message, None]:
         """Generate an asynchronous stream of agent responses and tool invocations.
 
@@ -427,11 +602,13 @@ class Agent:
 
         Args:
             messages: Initial conversation messages.
-            stream: Boolean flag for whether to yield partial message chunks from the inner API or not.
-            allow_tool: Boolean flag for whether to fully interpret tool calls/results in the history.
-            force_tool: Boolean flag for whether to force the use of tools.
-            thinking_budget: Budget for thinking tokens.
-            **kwargs: Additional keyword arguments to pass to the client.
+            stream: Whether to stream the response.
+            llm_exception_callback: Optional callback to determine retry behavior
+                when LLM exceptions occur. Receives the exception and current LLM index,
+                returns True to retry with next client, False to raise the exception.
+            **run_args: Additional arguments including allow_tool, force_tool,
+                thinking_budget, model, temperature, raise_on_null_content and other
+                keyword arguments to pass to the client.
 
         Yields:
             Message: Each message in the conversation, including tool uses and results.
@@ -442,9 +619,6 @@ class Agent:
         Raises:
             ValueError: If no response received from LLM, modified messages not available, or tool call did not yield result.
         """
-        # Add the agent instance.
-        kwargs["agent"] = self
-
         # Run in a loop.
         while True:
             # Get the response and yield
@@ -455,11 +629,9 @@ class Agent:
 
             async for output in self.step(
                 messages,
-                stream=stream,
-                allow_tool=allow_tool,
-                force_tool=force_tool,
-                thinking_budget=thinking_budget,
-                **kwargs,
+                stream,
+                llm_exception_callback=llm_exception_callback,
+                **run_args,
             ):
                 # This is a partial message chunk.
                 if isinstance(output, MessageContent):
@@ -477,7 +649,7 @@ class Agent:
                     response = output
 
             if response is None:
-                raise ValueError(f"No response received from the LLM {self.llm}.")
+                raise ValueError(f"No response received from the LLM {self.llms}.")
 
             if modified_messages is None:
                 raise ValueError("Modified messages are not available")
