@@ -6,14 +6,14 @@ import os
 import time
 from collections.abc import AsyncGenerator, Sequence
 from dataclasses import dataclass, field
-from typing import Any
+from typing import Any, Unpack
 
 from anthropic import AsyncAnthropic, AsyncAnthropicBedrock
 from anthropic.types import ToolUseBlock
 from anthropic.types import Usage as AnthropicUsage
 from pydantic import BaseModel
 
-from flexai.llm.client import Client
+from flexai.llm.client import AgentRunArgs, Client, PartialAgentRunArgs, with_defaults
 from flexai.message import (
     AIMessage,
     DataBlock,
@@ -27,7 +27,7 @@ from flexai.message import (
     Usage,
     UserMessage,
 )
-from flexai.tool import Tool
+from flexai.tool import Tool, ToolType
 
 
 def get_tool_call(tool_use: ToolUseBlock) -> ToolCall:
@@ -87,41 +87,41 @@ class AnthropicClient(Client):
     # Extra headers to include in the request.
     extra_headers: dict = field(default_factory=dict)
 
+    def _complete_args_with_defaults(
+        self, **run_args: Unpack[PartialAgentRunArgs]
+    ) -> AgentRunArgs:
+        defaults = with_defaults(**run_args)
+        if "temperature" not in run_args:
+            defaults["temperature"] = 1.0
+        if "force_tool" not in run_args:
+            defaults["force_tool"] = True
+        return defaults
+
     async def get_chat_response(
         self,
         messages: list[Message],
-        *,
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        thinking_budget: int | None = None,
-        temperature: float = 1.0,
-        force_tool: bool = True,
-        allow_tool: bool = True,
-        **kwargs,
+        tools: Sequence[Tool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
     ) -> AIMessage:
         if not messages:
             raise ValueError(
                 "Anthropic requires at least one messages to process your request."
             )
 
+        run_args = self._complete_args_with_defaults(**input_args)
+
         # If we were sent a model, call instead get_structured_response.
-        if "model" in kwargs:
+        if run_args["model"] is not None:
             # Extract the model parameter and remove it from kwargs to avoid recursion
-            model_class = kwargs.pop("model")
-            (
-                structured_response,
-                usage_info,
-            ) = await self._get_structured_response_with_usage(
+            model = run_args["model"]
+            run_args["model"] = None  # Prevent recursion
+            return await self._get_structured_response(
                 messages,
-                model_class,
+                model_to_use=model,
                 system=system,
                 tools=tools,
-                **kwargs,
-            )
-            # Convert structured response to AIMessage with the model as content
-            return AIMessage(
-                content=str(structured_response),
-                usage=usage_info,
+                **run_args,
             )
 
         # Send the messages to the model and get the response.
@@ -130,15 +130,9 @@ class AnthropicClient(Client):
             messages=messages,
             system=system,
             tools=tools,
-            temperature=temperature,
-            force_tool=force_tool,
-            allow_tool=allow_tool,
-            thinking_budget=thinking_budget,
             stream=False,
+            **run_args,
         )
-        agent = kwargs.get("agent")
-        if agent:
-            params.update(agent.config.extra_llm_params)
         response = await self.client.messages.create(**params)
         generation_time = time.time() - start
 
@@ -163,20 +157,17 @@ class AnthropicClient(Client):
     async def stream_chat_response(
         self,
         messages: list[Message],
-        *,
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        allow_tool: bool = True,
-        thinking_budget: int | None = None,
-        temperature: float = 1.0,
-        force_tool: bool = True,
-        **kwargs,
+        tools: Sequence[Tool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
     ) -> AsyncGenerator[MessageContent | AIMessage, None]:
         if not messages:
             raise ValueError(
-                "Anthropic requires at least one messages to process your request."
+                "Anthropic requires at least one message to process your request."
             )
 
+        run_args = self._complete_args_with_defaults(**input_args)
+
         # Initialize variables to store the tool information.
         tool_call: ToolCall | None = None
         text_block: TextBlock | None = None
@@ -190,15 +181,9 @@ class AnthropicClient(Client):
             messages=messages,
             system=system,
             tools=tools,
-            temperature=temperature,
-            force_tool=force_tool,
-            allow_tool=allow_tool,
-            thinking_budget=thinking_budget,
             stream=True,
+            **run_args,
         )
-        agent = kwargs.get("agent")
-        if agent:
-            params.update(agent.config.extra_llm_params)
         response_stream = await self.client.messages.create(**params)
         async for chunk in response_stream:
             # Add the usage.
@@ -307,24 +292,19 @@ class AnthropicClient(Client):
         self,
         messages: list[Message],
         system: str | SystemMessage,
-        tools: list[Tool] | None,
-        temperature: float,
-        force_tool: bool,
-        allow_tool: bool,
+        tools: Sequence[Tool] | None,
         stream: bool,
-        thinking_budget: int | None = None,
+        **run_args: Unpack[AgentRunArgs],
     ) -> dict:
         """Get the common params to send to the model.
 
         Args:
             messages: The messages to send to the model.
-            system: The system message to send to the model.
-            tools: The tools to send to the model.
-            temperature: The temperature to use for the model.
-            force_tool: Whether to force the model to use the tools.
-            allow_tool: Whether to allow tool calls in the content.
+            system: The system message or string.
+            tools: List of available tools.
             stream: Whether to stream the response.
-            thinking_budget: How many tokens are in the thinking budget (0 if it should be disabled).
+            **run_args: Additional arguments including temperature, force_tool,
+                allow_tool, thinking_budget and other run parameters.
 
         Returns:
             The common params to send to the model.
@@ -333,6 +313,12 @@ class AnthropicClient(Client):
         if isinstance(system, str):
             system = SystemMessage([TextBlock(system)])
 
+        # Extract the run arguments we care about
+        temperature = run_args["temperature"]
+        thinking_budget = run_args["thinking_budget"]
+        allow_tool = run_args["allow_tool"]
+        force_tool = run_args["force_tool"]
+
         thinking_args = (
             {"thinking": {"type": "disabled"}}
             if thinking_budget == 0
@@ -341,7 +327,7 @@ class AnthropicClient(Client):
             else {}
         )
 
-        kwargs = {
+        params = {
             "max_tokens": self.max_tokens,
             "messages": self._format_content(messages, allow_tool),
             "model": self.model,
@@ -357,50 +343,52 @@ class AnthropicClient(Client):
         if system_content and any(
             content.get("text", "").strip() for content in system_content
         ):
-            kwargs["system"] = system_content
+            params["system"] = system_content
 
         # If tools are provided, force the model to use them (for now).
         if tools:
-            kwargs["tools"] = sorted(
+            params["tools"] = sorted(
                 [self.format_tool(tool) for tool in tools], key=lambda x: x["name"]
             )
             if force_tool:
-                kwargs["tool_choice"] = {"type": "any"}
+                params["tool_choice"] = {"type": "any"}
         else:
-            kwargs["tools"] = []
+            params["tools"] = []
 
         if stream:
-            kwargs["stream"] = True
+            params["stream"] = True
 
         # Add cache control to the params.
-        return self._add_cache_control(kwargs)
+        return self._add_cache_control(params)
 
-    async def _get_structured_response_with_usage(
+    async def _get_structured_response(
         self,
         messages: list[Message],
-        model: type[BaseModel],
+        model_to_use: type[BaseModel],
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        temperature: float = 1.0,
-        **kwargs,
-    ) -> tuple[BaseModel, Usage]:
-        """Get the structured response from the chat model with usage tracking.
+        tools: Sequence[Tool] | None = None,
+        **run_args: Unpack[AgentRunArgs],
+    ) -> AIMessage:
+        """For anthropic, we have no better option than some funny retry logic to force a structured response. So that is pulled out into this function.
 
         Args:
-            messages: The messages to send to the model.
-            model: The model to use for the response.
-            system: Optional system message to set the behavior of the AI.
-            tools: Tools to use in the response.
-            temperature: The temperature to use for the model.
-            kwargs: Additional keyword arguments to pass to the model.
+            messages: List of messages for the conversation
+            model_to_use: The BaseModel type to use for structured response validation
+            system: System message or prompt
+            tools: Optional list of tools available to the model
+            **run_args: Additional arguments for the agent run
 
         Returns:
-            A tuple of (structured response, usage information).
+            AIMessage: The structured response from the model
 
         Raises:
-            TypeError: If the response is not a string.
+            TypeError: If the response is not a string
+            ValueError: If model parameter is provided in run_args (to avoid recursion)
         """
-        schema = model.model_json_schema()
+        if run_args["model"] is not None:
+            raise ValueError("model parameter should be None to avoid recursion.")
+
+        schema = model_to_use.model_json_schema()
         system = f"""{system}
 Return your answer according to the 'properties' of the following schema:
 {schema}
@@ -408,19 +396,23 @@ Return only the JSON object with the properties filled in.
 Do not include anything in your response other than the JSON object.
 Do not begin your response with ```json or end it with ```.
 """
+        run_args["model"] = None  # Prevent recursion
         response = await self.get_chat_response(
             messages,
             system=system,
             tools=tools,
-            temperature=temperature,
-            force_tool=False,
-            **kwargs,
+            **run_args,
         )
         content = response.content
         try:
             if not isinstance(content, str):
                 raise TypeError("The response is not a string.")
-            return model.model_validate_json(content), response.usage
+
+            # Try to parse the response as a sanity check.
+            model_to_use.model_validate_json(content)
+
+            return response
+
         except Exception as e:
             # Try again, printing the exception.
             messages = [
@@ -430,46 +422,25 @@ Do not begin your response with ```json or end it with ```.
                     f"There was an error while parsing. Make sure to only include the JSON. Error: {e}"
                 ),
             ]
-            return await self._get_structured_response_with_usage(
+            return await self._get_structured_response(
                 messages,
-                model=model,
+                model_to_use=model_to_use,
                 system=system,
                 tools=tools,
-                temperature=temperature,
-                **kwargs,
+                **run_args,
             )
 
-    async def get_structured_response(
-        self,
-        messages: list[Message],
-        model: type[BaseModel],
-        system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        temperature: float = 1.0,
-        **kwargs,
-    ) -> BaseModel:
-        """Get the structured response from the chat model.
-
-        Args:
-            messages: The messages to send to the model.
-            model: The model to use for the response.
-            system: Optional system message to set the behavior of the AI.
-            tools: Tools to use in the response.
-            temperature: The temperature to use for the model.
-            kwargs: Additional keyword arguments to pass to the model.
-
-        Returns:
-            The structured response from the model.
-        """
-        structured_response, _ = await self._get_structured_response_with_usage(
-            messages,
-            model,
-            system=system,
-            tools=tools,
-            temperature=temperature,
-            **kwargs,
-        )
-        return structured_response
+    @staticmethod
+    def format_type(arg_type: ToolType):
+        if isinstance(arg_type, tuple):
+            return {
+                "anyOf": [
+                    AnthropicClient.format_type(sub_type) for sub_type in arg_type
+                ]
+            }
+        return {
+            "type": arg_type,
+        }
 
     @staticmethod
     def format_tool(tool: Tool) -> dict:
@@ -492,9 +463,9 @@ Do not begin your response with ```json or end it with ```.
             "properties": {},
         }
         for param_name, param_type in tool.params:
-            input_schema["properties"][param_name] = {
-                "type": param_type,
-            }
+            input_schema["properties"][param_name] = AnthropicClient.format_type(
+                param_type
+            )
 
         return {
             "name": tool.name,
@@ -591,7 +562,7 @@ Do not begin your response with ```json or end it with ```.
         value: Message | Sequence[Message],
         allow_tool: bool = True,
     ) -> list[dict[str, Any]]:
-        """Format the message content for the Anthropic model.
+        """Format the message content for the Anthropic API.
 
         Args:
             value: The value to format.
