@@ -2,14 +2,83 @@ from __future__ import annotations
 
 import functools
 from abc import ABC, abstractmethod
-from collections.abc import AsyncGenerator, AsyncIterator, Awaitable, Callable
+from collections.abc import AsyncGenerator, AsyncIterator, Awaitable, Callable, Sequence
 from dataclasses import dataclass, field
-from typing import Any, TypeVar
+from typing import Any, TypedDict, TypeVar, Unpack
 
 from flexai.llm.exceptions import LLMException, map_exception
 from flexai.message import AIMessage, Message, MessageContent, SystemMessage
 from flexai.tool import Tool
 
+
+class AgentRunArgs(TypedDict):
+    """Any arguments passed to an agent run that control the response, aside from the base parameters (message list, system prompts, tools)."""
+
+    # Whether we are forcing the AI to call a tool or not.
+    force_tool: bool
+
+    # Whether to format past tool calls literally in the input or not (to avoid some risk of misinterpretation by the AI).
+    allow_tool: bool
+
+    # Allowed thinking budget tokens for the AI. 0 if we want to disable thinking. None if we don't care (and want to use the provider's default).
+    thinking_budget: int | None
+
+    # Whether to include the LLM's internal thoughts in the response, wherever applicable.
+    include_thoughts: bool
+
+    # Sampling temperature for the model. None if to use the provider's default.
+    temperature: float | None
+
+    # A (usually pydantic) model, in case we want to force the AI's response to be structured. None if we don't need a structured response.
+    model: Any | None
+
+    # Whether to raise an exception when the AI responds with null content, or to just have the default behavior (which is to silently exit the loop).
+    raise_on_null_content: bool
+
+    # Whether to allow text responses from the model.
+    allow_text: bool
+
+    # Whether to allow image responses from the model.
+    allow_image: bool
+
+
+class PartialAgentRunArgs(TypedDict, total=False):
+    """AgentRunArgs, but some fields are allowed to be unspecified."""
+
+    force_tool: bool
+    allow_tool: bool
+    thinking_budget: int | None
+    include_thoughts: bool
+    temperature: float | None
+    model: Any | None
+    raise_on_null_content: bool
+    allow_text: bool
+    allow_image: bool
+
+
+def with_defaults(**kwargs: Unpack[PartialAgentRunArgs]) -> AgentRunArgs:
+    """Create AgentRunArgs with default values filled in.
+
+    Args:
+        **kwargs: Partial AgentRunArgs to override defaults
+
+    Returns:
+        AgentRunArgs: Complete AgentRunArgs with defaults filled in
+    """
+    defaults: AgentRunArgs = {
+        "force_tool": False,
+        "allow_tool": True,
+        "thinking_budget": None,
+        "include_thoughts": False,
+        "temperature": None,
+        "model": None,
+        "raise_on_null_content": False,
+        "allow_text": True,
+        "allow_image": False,
+    }
+    return {**defaults, **kwargs}
+
+
 T = TypeVar("T")
 
 
@@ -81,15 +150,28 @@ class Client(ABC):
     # Provider name (e.g., "openai", "anthropic", "gemini")
     provider: str = field(kw_only=True)
 
+    def _complete_args_with_defaults(
+        self, **run_args: Unpack[PartialAgentRunArgs]
+    ) -> AgentRunArgs:
+        """Fill in missing run arguments with default values.
+
+        Default behavior is use dummy values, but this can be overriden by different clients.
+
+        Args:
+            run_args: PartialAgentRunArgs with some fields possibly missing
+
+        Returns:
+            Complete AgentRunArgs with defaults filled in
+        """
+        return with_defaults(**run_args)
+
     @abstractmethod
     async def get_chat_response(
         self,
         messages: list[Message],
-        *,
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        force_tool: bool = True,
-        thinking_budget: int | None = None,
+        tools: Sequence[Tool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
     ) -> AIMessage:
         """Retrieve a response from the chat model.
 
@@ -97,8 +179,8 @@ class Client(ABC):
             messages: Conversation history to send to the model.
             system: Optional system message to set the behavior of the AI.
             tools: Optional list of tools available for the model to use.
-            force_tool: Boolean flag for whether to fully interpret tool calls/results in the history.
-            thinking_budget: Budget for thinking tokens.
+            **input_args: Additional arguments including force_tool, thinking_budget,
+                and other run parameters.
 
         Returns:
             A list of AI-generated messages in response to the input.
@@ -112,12 +194,9 @@ class Client(ABC):
     def stream_chat_response(
         self,
         messages: list[Message],
-        *,
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        allow_tool: bool = True,
-        force_tool: bool = True,
-        thinking_budget: int | None = None,
+        tools: Sequence[Tool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
     ) -> AsyncIterator[MessageContent | AIMessage]:
         """Stream the response from the chat model in real-time.
 
@@ -125,9 +204,8 @@ class Client(ABC):
             messages: Conversation history to send to the model.
             system: Optional system message to set the behavior of the AI.
             tools: Optional list of tools available for the model to use.
-            allow_tool: Boolean flag for whether to fully interpret tool calls/results in the history.
-            force_tool: Boolean flag for whether to fully interpret tool calls/results in the history.
-            thinking_budget: Budget for thinking tokens.
+            **input_args: Additional arguments including allow_tool, force_tool,
+                thinking_budget and other run parameters.
 
         Yields:
             AI-generated messages as they are generated by the model.
