@@ -8,7 +8,7 @@ import uuid
 from collections.abc import AsyncGenerator, Sequence
 from dataclasses import InitVar, dataclass, field
 from pathlib import Path
-from typing import Any
+from typing import Any, Unpack
 
 from google import genai
 from google.genai import types
@@ -20,10 +20,9 @@ from google.genai.types import (
 )
 from google.genai.types import Type as DataType
 from google.oauth2 import service_account
-from pydantic import BaseModel
 
-from flexai.llm.client import Client
-from flexai.llm.exceptions import NoContentException, StructuredResponseException
+from flexai.llm.client import AgentRunArgs, Client, PartialAgentRunArgs, with_defaults
+from flexai.llm.exceptions import NoContentException
 from flexai.message import (
     AIMessage,
     DataBlock,
@@ -39,7 +38,7 @@ from flexai.message import (
     URLContextBlock,
     Usage,
 )
-from flexai.tool import Tool
+from flexai.tool import Tool, ToolType
 
 
 def get_tool_call(function_call) -> ToolCall:
@@ -288,6 +287,24 @@ class GeminiClient(Client):
             raise ValueError("No Gemini Client Configured.")
         return self._client
 
+    def _complete_args_with_defaults(
+        self, **run_args: Unpack[PartialAgentRunArgs]
+    ) -> AgentRunArgs:
+        defaults = with_defaults(**run_args)
+        if "force_tool" not in run_args:
+            defaults["force_tool"] = True
+        if run_args.get("thinking_budget") is None:
+            defaults["thinking_budget"] = self.default_thinking_budget
+        return defaults
+
+    @staticmethod
+    def format_type(arg_type: ToolType) -> Schema:
+        if isinstance(arg_type, tuple):
+            return Schema(
+                any_of=[GeminiClient.format_type(sub_type) for sub_type in arg_type]
+            )
+        return Schema(type=DataType(arg_type))
+
     @staticmethod
     def format_tool(tool: Tool) -> FunctionDeclaration:
         """Convert a FlexAI Tool to a Gemini FunctionDeclaration.
@@ -304,7 +321,7 @@ class GeminiClient(Client):
             parameters=Schema(
                 type=DataType.OBJECT,
                 properties={
-                    param_name: Schema(type=DataType(param_type))
+                    param_name: GeminiClient.format_type(param_type)
                     for param_name, param_type in tool.params
                 },
             ),
@@ -405,17 +422,18 @@ class GeminiClient(Client):
             }
         if isinstance(content, ToolResult):
             formatted_result = content.result
-            if isinstance(formatted_result, DataBlock):
-                formatted_result = TextBlock(text=json.dumps(formatted_result.data))
 
             if isinstance(formatted_result, str):
-                formatted_result = {
-                    "result": formatted_result,
-                }
-            elif not isinstance(formatted_result, dict):
+                formatted_result = TextBlock(
+                    text=formatted_result,
+                )
+
+            try:
                 formatted_result = cls._format_message_content(
                     formatted_result, name_context=name_context
                 )
+            except ValueError:
+                formatted_result = {"result": str(formatted_result)}
 
             if content.tool_call_id not in name_context:
                 raise ValueError(
@@ -434,36 +452,32 @@ class GeminiClient(Client):
         self,
         messages: list[Message],
         system: str | SystemMessage,
-        tools: list[Tool] | None,
-        force_tool: bool,
-        include_thoughts: bool,
-        thinking_budget: int | None,
-        use_url_context: bool = False,
-        use_google_search: bool = False,
-        google_search_dynamic_threshold: float | None = None,
-        allow_text=True,
-        allow_image=False,
-        **kwargs,
+        tools: Sequence[Tool] | None,
+        stream: bool,
+        **run_args: Unpack[AgentRunArgs],
     ):
         """Build parameters for Gemini API request.
 
         Args:
-            messages: List of conversation messages.
-            system: System message or string.
-            tools: Available tools for the model to use.
-            force_tool: Whether to force tool usage.
-            include_thoughts: Whether to include thinking in response.
-            thinking_budget: Token budget for thinking (None for unlimited).
-            use_url_context: Whether to enable URL context retrieval.
-            use_google_search: Whether to enable Google Search integration.
-            google_search_dynamic_threshold: Threshold for dynamic search retrieval.
-            allow_text: Whether to allow text content in responses.
-            allow_image: Whether to allow image content in responses.
-            kwargs: Additional parameters including structured output models.
+            messages: The messages to send to the model.
+            system: The system message or string.
+            tools: List of available tools.
+            stream: Whether to stream the response.
+            **run_args: Additional arguments including force_tool, include_thoughts,
+                thinking_budget, use_url_context, use_google_search,
+                google_search_dynamic_threshold and other parameters.
 
         Returns:
             Dictionary of formatted parameters for Gemini API call.
         """
+        # Extract run arguments that we care about
+        force_tool = run_args["force_tool"]
+        include_thoughts = run_args["include_thoughts"]
+        model = run_args["model"]
+        thinking_budget = run_args["thinking_budget"]
+        allow_text = run_args["allow_text"]
+        allow_image = run_args["allow_image"]
+
         name_context = {}
 
         formatted_messages = [
@@ -501,35 +515,6 @@ class GeminiClient(Client):
             config_args["thinking_config"] = types.ThinkingConfig(**thinking_args)
 
         gemini_tools = []
-        if use_url_context:
-            gemini_tools.append(types.Tool(url_context=types.UrlContext()))
-
-        if use_google_search:
-            # Check if we're using Gemini 2.0+ models (recommended google_search tool)
-            # or Gemini 1.5 models (legacy google_search_retrieval tool)
-            model_version = self.model.lower()
-            if any(ver in model_version for ver in ["2.0", "2.5"]):
-                # Use the new google_search tool for Gemini 2.0+ models
-                gemini_tools.append(types.Tool(google_search=types.GoogleSearch()))
-            # Use the legacy google_search_retrieval tool for Gemini 1.5 models
-            elif google_search_dynamic_threshold is not None:
-                # Dynamic mode with threshold
-                retrieval_config = types.DynamicRetrievalConfig(
-                    mode=types.DynamicRetrievalConfigMode.MODE_DYNAMIC,
-                    dynamic_threshold=google_search_dynamic_threshold,
-                )
-                gemini_tools.append(
-                    types.Tool(
-                        google_search_retrieval=types.GoogleSearchRetrieval(
-                            dynamic_retrieval_config=retrieval_config
-                        )
-                    )
-                )
-            else:
-                # Default mode (always search)
-                gemini_tools.append(
-                    types.Tool(google_search_retrieval=types.GoogleSearchRetrieval())
-                )
 
         if tools:
             # Create a formatted tool list
@@ -554,11 +539,11 @@ class GeminiClient(Client):
                 }
             )
 
-        if "model" in kwargs:
+        if model is not None:
             config_args.update(
                 {
                     "response_mime_type": "application/json",
-                    "response_schema": kwargs["model"],
+                    "response_schema": model,
                 }
             )
 
@@ -584,18 +569,9 @@ class GeminiClient(Client):
     async def get_chat_response(
         self,
         messages: list[Message],
-        *,
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        force_tool: bool = True,
-        thinking_budget: int | None = None,
-        include_thoughts: bool = False,
-        use_url_context: bool = False,
-        use_google_search: bool = False,
-        google_search_dynamic_threshold: float | None = None,
-        allow_text: bool = True,
-        allow_image: bool = False,
-        **kwargs,
+        tools: Sequence[Tool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
     ) -> AIMessage:
         """Get a chat response from the Gemini model.
 
@@ -603,15 +579,9 @@ class GeminiClient(Client):
             messages: List of conversation messages.
             system: System message to set AI behavior.
             tools: Available tools for the model to use.
-            force_tool: Whether to force tool usage when tools are available.
-            thinking_budget: Token budget for thinking (None uses default).
-            include_thoughts: Whether to include thinking in the response.
-            use_url_context: Whether to enable URL context retrieval.
-            use_google_search: Whether to enable Google Search integration.
-            google_search_dynamic_threshold: Threshold for dynamic search retrieval.
-            allow_text: Whether to allow text content in responses.
-            allow_image: Whether to allow image content in responses.
-            kwargs: Additional parameters including structured output models.
+            **input_args: Additional arguments including force_tool, thinking_budget,
+                include_thoughts, etc.
+                google_search_dynamic_threshold and other parameters.
 
         Returns:
             An AIMessage containing the model's response and usage information.
@@ -620,29 +590,14 @@ class GeminiClient(Client):
             ValueError: If the model doesn't respond with usage metadata.
             NoContentException: If the model doesn't return any content.
         """
-        extra_param_args = {}
-        if "model" in kwargs:
-            extra_param_args["model"] = kwargs["model"]
+        run_args = self._complete_args_with_defaults(**input_args)
 
-        # If this client has a default thinking budget set, use that if one wasn't specified here
-        thinking_budget = (
-            thinking_budget
-            if thinking_budget is not None
-            else self.default_thinking_budget
-        )
         params = self._get_params(
             messages=messages,
             system=system,
             tools=tools,
-            force_tool=force_tool,
-            thinking_budget=thinking_budget,
-            include_thoughts=include_thoughts,
-            use_url_context=use_url_context,
-            use_google_search=use_google_search,
-            google_search_dynamic_threshold=google_search_dynamic_threshold,
-            allow_text=allow_text,
-            allow_image=allow_image,
-            **extra_param_args,
+            stream=False,
+            **run_args,
         )
         start = time.time()
         response_object = await self.client.models.generate_content(
@@ -661,17 +616,27 @@ class GeminiClient(Client):
             cache_write_tokens=0,  # Currently not accounted for
             generation_time=time.time() - start,
         )
-        # Handle this case
+
+        # Argument to raise if no content is returned
+        raise_on_null_content = run_args["raise_on_null_content"]
+
         if not response_object.candidates or not response_object.candidates[0].content:
-            raise NoContentException(
-                message="Gemini did not respond with any content, and the candidates are null.",
-                provider="gemini",
-            )
+            if raise_on_null_content:
+                raise NoContentException(
+                    message="Gemini did not respond with any content, and the candidates are null.",
+                    provider="gemini",
+                )
+            return AIMessage(content=[], usage=usage)
+
         response_content_parts = response_object.candidates[0].content.parts
         if not response_content_parts:
-            raise NoContentException(
-                message="Gemini did not respond with any content.", provider="gemini"
-            )
+            if raise_on_null_content:
+                raise NoContentException(
+                    message="Gemini did not respond with any content.",
+                    provider="gemini",
+                )
+            return AIMessage(content=[], usage=usage)
+
         formatted_content_parts: list[
             ImageBlock
             | TextBlock
@@ -686,23 +651,6 @@ class GeminiClient(Client):
                 list(self._extract_content_from_part_object(part))
             )
 
-        # Add URL context block if metadata is present
-        if response_object.candidates and len(response_object.candidates) > 0:
-            url_context_metadata = getattr(
-                response_object.candidates[0], "url_context_metadata", None
-            )
-            if url_context_block := self._extract_url_context_block(
-                url_context_metadata
-            ):
-                formatted_content_parts.append(url_context_block)
-
-            # Add grounding block if metadata is present
-            grounding_metadata = getattr(
-                response_object.candidates[0], "grounding_metadata", None
-            )
-            if grounding_block := self._extract_grounding_block(grounding_metadata):
-                formatted_content_parts.append(grounding_block)
-
         return AIMessage(
             content=formatted_content_parts,
             usage=usage,
@@ -711,17 +659,9 @@ class GeminiClient(Client):
     async def stream_chat_response(
         self,
         messages: list[Message],
-        *,
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        allow_tool: bool = True,
-        force_tool: bool = True,
-        thinking_budget: int | None = None,
-        include_thoughts: bool = False,
-        use_url_context: bool = False,
-        use_google_search: bool = False,
-        google_search_dynamic_threshold: float | None = None,
-        **kwargs,
+        tools: Sequence[Tool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
     ) -> AsyncGenerator[MessageContent | AIMessage, None]:
         """Stream a chat response from the Gemini model.
 
@@ -729,14 +669,10 @@ class GeminiClient(Client):
             messages: List of conversation messages.
             system: System message to set AI behavior.
             tools: Available tools for the model to use.
-            allow_tool: Whether to allow tool usage (kept for compatibility).
-            force_tool: Whether to force tool usage when tools are available.
-            thinking_budget: Token budget for thinking (None uses default).
-            include_thoughts: Whether to include thinking in the response.
-            use_url_context: Whether to enable URL context retrieval.
-            use_google_search: Whether to enable Google Search integration.
-            google_search_dynamic_threshold: Threshold for dynamic search retrieval.
-            kwargs: Additional parameters including structured output models.
+            **input_args: Additional arguments including force_tool, thinking_budget,
+                include_thoughts, use_url_context, use_google_search,
+                google_search_dynamic_threshold, model, temperature,
+                raise_on_null_content and other parameters.
 
         Yields:
             TextBlock, ThoughtBlock, ToolCall content as they're generated,
@@ -745,28 +681,15 @@ class GeminiClient(Client):
         Raises:
             ValueError: If the model doesn't respond with usage metadata.
         """
-        extra_param_args = {}
-        if "model" in kwargs:
-            extra_param_args["model"] = kwargs["model"]
+        run_args = self._complete_args_with_defaults(**input_args)
 
-        # If this client has a default thinking budget set, use that if one wasn't specified here
-        thinking_budget = (
-            thinking_budget
-            if thinking_budget is not None
-            else self.default_thinking_budget
-        )
         usage = Usage()
         params = self._get_params(
             messages=messages,
             system=system,
             tools=tools,
-            force_tool=force_tool,
-            thinking_budget=thinking_budget,
-            include_thoughts=include_thoughts,
-            use_url_context=use_url_context,
-            use_google_search=use_google_search,
-            google_search_dynamic_threshold=google_search_dynamic_threshold,
-            **extra_param_args,
+            stream=True,
+            **run_args,
         )
         start = time.time()
         response_object = await self.client.models.generate_content_stream(
@@ -776,36 +699,19 @@ class GeminiClient(Client):
         total_content_list: list[
             TextBlock | ThoughtBlock | ToolCall | URLContextBlock | GroundingBlock
         ] = []
-        url_context_metadata = None
-        grounding_metadata = None
 
         async for chunk in response_object:
             # Handle this case
             if not chunk.candidates or not chunk.candidates[0].content:
                 continue
 
-            chunk_parts = chunk.candidates[0].content.parts
-
-            if chunk.candidates and len(chunk.candidates) > 0:
-                chunk_url_context_metadata = getattr(
-                    chunk.candidates[0], "url_context_metadata", None
-                )
-                if chunk_url_context_block := self._extract_url_context_block(
-                    chunk_url_context_metadata
-                ):
-                    yield chunk_url_context_block
-                chunk_grounding_metadata = getattr(
-                    chunk.candidates[0], "grounding_metadata", None
-                )
-                if chunk_grounding_block := self._extract_grounding_block(
-                    chunk_grounding_metadata
-                ):
-                    yield chunk_grounding_block
-
             usage_metadata = chunk.usage_metadata
             if not usage_metadata:
                 raise ValueError("Gemini did not respond with any usage metadata.")
 
+            # Obtaining chunk parts
+            chunk_parts = chunk.candidates[0].content.parts
+
             usage += get_usage_block(usage_metadata)
 
             if isinstance(chunk_parts, list):
@@ -826,14 +732,6 @@ class GeminiClient(Client):
         if text_buffer:
             total_content_list.append(text_buffer)
 
-        # Add URL context block if metadata is present
-        if url_context_block := self._extract_url_context_block(url_context_metadata):
-            total_content_list.append(url_context_block)
-
-        # Add grounding block if metadata is present
-        if grounding_block := self._extract_grounding_block(grounding_metadata):
-            total_content_list.append(grounding_block)
-
         yield AIMessage(
             content=total_content_list,
             usage=usage,
@@ -984,63 +882,6 @@ class GeminiClient(Client):
 
         return content
 
-    async def get_structured_response(
-        self,
-        messages: list[Message],
-        model: type[BaseModel],
-        system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        thinking_budget: int | None = None,
-        include_thoughts: bool = False,
-        use_url_context: bool = False,
-        use_google_search: bool = False,
-        google_search_dynamic_threshold: float | None = None,
-        **kwargs,
-    ) -> BaseModel:
-        """Get the structured response from the chat model.
-
-        Args:
-            messages: The messages to send to the model.
-            model: The model to use for the response.
-            system: Optional system message to set the behavior of the AI.
-            tools: Tools to use in the response.
-            thinking_budget: The thinking budget for this request (0 if it should be disabled).
-            include_thoughts: Whether to include thoughts in the response.
-            use_url_context: Whether to enable URL context for accessing web content.
-            use_google_search: Whether to enable Google Search for grounding.
-            google_search_dynamic_threshold: Dynamic threshold for Google Search.
-            kwargs: Additional keyword arguments to pass to the model.
-
-        Returns:
-            The structured response from the model.
-
-        Raises:
-            StructuredResponseException: If the response cannot be parsed as the expected model.
-        """
-        response = await self.get_chat_response(
-            messages,
-            system=system,
-            tools=tools,
-            force_tool=False,
-            thinking_budget=thinking_budget,
-            include_thoughts=include_thoughts,
-            use_url_context=use_url_context,
-            use_google_search=use_google_search,
-            google_search_dynamic_threshold=google_search_dynamic_threshold,
-            model=model.model_json_schema(),
-            **kwargs,
-        )
-
-        try:
-            content = self._extract_text_content(response.content)
-            return model.model_validate_json(content)
-        except Exception as e:
-            raise StructuredResponseException(
-                message=f"Failed to parse structured response as {model.__name__}: {e}",
-                provider="gemini",
-                original_exception=e,
-            ) from e
-
     def get_endpoint_info(self) -> dict[str, str | bool]:
         """Get information about the current endpoint configuration.
 
