@@ -6,12 +6,12 @@ import os
 import time
 from collections.abc import AsyncGenerator, Sequence
 from dataclasses import InitVar, dataclass, field
-from typing import Any, TypeVar
+from typing import Any, TypeVar, Unpack
 
 from openai import AsyncOpenAI
 from pydantic import BaseModel
 
-from flexai.llm.client import Client
+from flexai.llm.client import AgentRunArgs, Client, PartialAgentRunArgs, with_defaults
 from flexai.message import (
     AIMessage,
     ImageBlock,
@@ -23,7 +23,7 @@ from flexai.message import (
     ToolResult,
     Usage,
 )
-from flexai.tool import Tool
+from flexai.tool import Tool, ToolType
 
 
 def get_tool_call(tool_use) -> ToolCall:
@@ -42,6 +42,21 @@ def get_tool_call(tool_use) -> ToolCall:
     )
 
 
+def get_usage_block(usage) -> Usage:
+    """Extract usage information from the OpenAI response.
+
+    Args:
+        usage: The usage object from OpenAI.
+
+    Returns:
+        A Usage object containing input tokens, output tokens, and generation time.
+    """
+    return Usage(
+        input_tokens=usage.prompt_tokens,
+        output_tokens=usage.completion_tokens,
+    )
+
+
 BASE_MODEL = TypeVar("BASE_MODEL", bound=BaseModel)
 
 
@@ -71,26 +86,41 @@ class OpenAIClient(Client):
             AsyncOpenAI(api_key=api_key, base_url=base_url),
         )
 
+    def _complete_args_with_defaults(
+        self, **run_args: Unpack[PartialAgentRunArgs]
+    ) -> AgentRunArgs:
+        defaults = with_defaults(**run_args)
+        if "temperature" not in run_args:
+            defaults["temperature"] = 1.0
+        return defaults
+
     async def get_chat_response(
         self,
         messages: list[Message],
-        *,
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        temperature: float = 1.0,
-        force_tool: bool = True,
-        allow_tool: bool = True,
-        **kwargs,
+        tools: Sequence[Tool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
     ) -> AIMessage:
+        run_args = self._complete_args_with_defaults(**input_args)
+
+        if run_args["model"] is not None:
+            model = run_args["model"]
+            run_args["model"] = None  # Clear model from run_args to avoid duplication
+            return await self._get_structured_response(
+                messages=messages,
+                model_to_use=model,
+                system=system,
+                tools=tools,
+                **run_args,
+            )
+
         # Send the messages to the model and get the response.
         params = self._get_params(
             messages=messages,
             system=system,
             tools=tools,
-            temperature=temperature,
-            force_tool=force_tool,
-            allow_tool=allow_tool,
             stream=False,
+            **run_args,
         )
         start = time.time()
         response = await self.client.chat.completions.create(**params)
@@ -99,38 +129,32 @@ class OpenAIClient(Client):
         # Parse out the tool uses from the response.
         message = response.choices[0].message
         tool_uses = [get_tool_call(message) for message in (message.tool_calls or [])]
+        usage = get_usage_block(response.usage)
+        usage.generation_time = generation_time
 
         # Get the content to return.
         content_to_return = tool_uses or message.content
         return AIMessage(
             content=content_to_return,
-            usage=Usage(
-                input_tokens=response.usage.prompt_tokens,
-                output_tokens=response.usage.completion_tokens,
-                generation_time=generation_time,
-            ),
+            usage=usage,
         )
 
     async def stream_chat_response(
         self,
         messages: list[Message],
-        *,
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        allow_tool: bool = True,
-        temperature: float = 1.0,
-        force_tool: bool = True,
-        **kwargs,
+        tools: Sequence[Tool] | None = None,
+        **input_args: Unpack[PartialAgentRunArgs],
     ) -> AsyncGenerator[MessageContent | AIMessage, None]:
+        run_args = self._complete_args_with_defaults(**input_args)
+
         stream = await self.client.chat.completions.create(
             **self._get_params(
                 messages=messages,
                 system=system,
                 tools=tools,
-                temperature=temperature,
-                force_tool=force_tool,
-                allow_tool=allow_tool,
                 stream=True,
+                **run_args,
             )
         )
         current_tool_call: ToolCall | None = None
@@ -177,11 +201,9 @@ class OpenAIClient(Client):
         self,
         messages: list[Message],
         system: str | SystemMessage,
-        tools: list[Tool] | None,
-        temperature: float,
-        force_tool: bool = False,
-        allow_tool: bool = False,
-        stream: bool = False,
+        tools: Sequence[Tool] | None,
+        stream: bool,
+        **run_args: Unpack[AgentRunArgs],
     ) -> dict:
         """Get the common params to send to the model.
 
@@ -189,14 +211,16 @@ class OpenAIClient(Client):
             messages: The messages to send to the model.
             system: The system message to send to the model.
             tools: The tools to send to the model.
-            temperature: The temperature to use for the model.
-            force_tool: unused
-            allow_tool: unused
-            stream: Whether we are streaming.
+            stream: Whether to stream the response.
+            **run_args: Additional arguments including temperature, force_tool,
+                allow_tool, stream and other run parameters.
 
         Returns:
             The common params to send to the model.
         """
+        # Extract the run arguments we care about.
+        temperature = run_args["temperature"]
+
         if isinstance(system, str):
             system = SystemMessage(system)
         kwargs = {
@@ -216,40 +240,59 @@ class OpenAIClient(Client):
 
         return kwargs
 
-    async def get_structured_response(
+    async def _get_structured_response(
         self,
         messages: list[Message],
-        model: type[BASE_MODEL],
+        model_to_use: type[BASE_MODEL],
         system: str | SystemMessage = "",
-        tools: list[Tool] | None = None,
-        temperature: float = 1.0,
-    ) -> BASE_MODEL:
+        tools: Sequence[Tool] | None = None,
+        **run_args: Unpack[AgentRunArgs],
+    ) -> AIMessage:
         """Get the structured response from the chat model.
 
         Args:
-            messages: The messages to send to the model.
-            model: The model to use for the response.
-            system: Optional system message to set the behavior of the AI.
-            tools: unused.
-            temperature: The temperature to use for the model.
+            messages: List of messages for the conversation
+            model_to_use: The BaseModel type to use for structured response parsing
+            system: System message or prompt
+            tools: Optional list of tools available to the model
+            **run_args: Additional arguments for the agent run
 
         Returns:
-            The structured response from the model.
+            AIMessage: A message containing a single instance of the structured response model
 
         Raises:
-            ValueError: If the response cannot be parsed.
+            ValueError: If the response fails to parse
         """
         # Send the messages to the model and get the response.
         response = await self.client.beta.chat.completions.parse(
             **self._get_params(
-                messages=messages, system=system, temperature=temperature, tools=[]
+                messages=messages,
+                system=system,
+                tools=tools,
+                stream=False,
+                **run_args,
             ),
-            response_format=model,
+            response_format=model_to_use,
         )
         result = response.choices[0].message.parsed
         if result is None:
             raise ValueError("Failed to parse the response.")
-        return result
+        usage = get_usage_block(response.usage)
+
+        return AIMessage(
+            content=result.model_dump_json(),
+            usage=usage,
+        )
+
+    @staticmethod
+    def format_type(arg_type: ToolType):
+        if isinstance(arg_type, tuple):
+            return {
+                "anyOf": [OpenAIClient.format_type(sub_type) for sub_type in arg_type]
+            }
+        return {
+            "type": arg_type,
+        }
 
     @staticmethod
     def format_tool(tool: Tool) -> dict:
@@ -266,9 +309,9 @@ class OpenAIClient(Client):
             "properties": {},
         }
         for param_name, param_type in tool.params:
-            input_schema["properties"][param_name] = {
-                "type": param_type,
-            }
+            input_schema["properties"][param_name] = OpenAIClient.format_type(
+                param_type
+            )
 
         return {
             "type": "function",
@@ -325,7 +368,7 @@ class OpenAIClient(Client):
         value: Message | Sequence[Message],
         allow_tool: bool = True,
     ) -> list[dict[str, Any]]:
-        """Format the message content for the Anthropic model.
+        """Format the message content for the OpenAI API.
 
         Args:
             value: The value to format.
@@ -392,7 +435,7 @@ class OpenAIClient(Client):
     def load_content(
         cls, content: str | list[dict[str, Any]]
     ) -> str | list[MessageContent]:
-        """Load the message content from the Anthropic model to dataclasses.
+        """Load the message content from the OpenAI API to dataclasses.
 
         Args:
             content: The content to load.
