@@ -396,13 +396,17 @@ class TestGeminiIntegration:
 
         test_url = "https://openai.com"
 
-        result = await gemini_structured_client.get_structured_response(
+        response = await gemini_structured_client.get_chat_response(
             messages=[
                 UserMessage(f"Analyze {test_url} and provide a structured summary.")
             ],
             model=UrlAnalysis,
             use_url_context=True,
         )
+        # Parse the JSON content into the Pydantic model
+        import json
+
+        result = UrlAnalysis.model_validate(json.loads(response.content))
 
         # Verify structured response
         assert isinstance(result, UrlAnalysis)
