@@ -14,12 +14,10 @@ from google.genai import types
 from flexai.llm.gemini import GeminiClient
 from flexai.message import (
     AIMessage,
-    GroundingBlock,
     ImageBlock,
     TextBlock,
     ThoughtBlock,
     ToolCall,
-    URLContextBlock,
     UserMessage,
 )
 from flexai.tool import Tool
@@ -718,368 +716,3 @@ class TestGeminiClient:
         assert streamed_content[0].id == "func_789"
         assert streamed_content[0].name == "example_function"
         assert streamed_content[0].input == {"a": "stream test", "b": 123}
-
-    @pytest.mark.asyncio
-    async def test_url_context_enabled(self, mock_client):
-        """Test URL context when enabled."""
-        response = create_mock_response([("text", "Content analyzed")])
-        response.candidates[0].url_context_metadata = {
-            "urls_accessed": ["https://example.com"]
-        }
-        mock_client.models.generate_content = AsyncMock(return_value=response)
-
-        with patch.object(GeminiClient, "__post_init__"):
-            client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "_client", mock_client)
-
-            result = await client.get_chat_response(
-                messages=[UserMessage(content="Analyze https://example.com")],
-                use_url_context=True,
-            )
-
-        # Verify URL context tool was added
-        call_args = mock_client.models.generate_content.call_args[1]
-        config = call_args["config"]
-        assert hasattr(config, "tools")
-        assert len(config.tools) == 1
-        assert hasattr(config.tools[0], "url_context")
-
-        # Verify URLContextBlock is included in content
-        url_context_blocks = [
-            content
-            for content in result.content
-            if isinstance(content, URLContextBlock)
-        ]
-        assert len(url_context_blocks) == 1
-        assert url_context_blocks[0].metadata["url_metadata"] == {
-            "urls_accessed": ["https://example.com"]
-        }
-
-    @pytest.mark.asyncio
-    async def test_url_context_disabled(self, mock_client):
-        """Test URL context when disabled (default)."""
-        response = create_mock_response([("text", "Regular response")])
-        # Explicitly set url_context_metadata to None when disabled
-        response.candidates[0].url_context_metadata = None
-        mock_client.models.generate_content = AsyncMock(return_value=response)
-
-        with patch.object(GeminiClient, "__post_init__"):
-            client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "_client", mock_client)
-
-            result = await client.get_chat_response(
-                messages=[UserMessage(content="Regular message")],
-                # use_url_context defaults to False
-            )
-
-        # Verify no tools were added
-        call_args = mock_client.models.generate_content.call_args[1]
-        config = call_args["config"]
-        if hasattr(config, "tools"):
-            assert config.tools is None
-
-        # Verify no URLContextBlock in content
-        url_context_blocks = [
-            content
-            for content in result.content
-            if isinstance(content, URLContextBlock)
-        ]
-        assert len(url_context_blocks) == 0
-
-    @pytest.mark.asyncio
-    async def test_google_search_enabled_gemini_25(self, mock_client):
-        """Test Google Search grounding with Gemini 2.5+ models (google_search tool)."""
-        response = create_mock_response(
-            [("text", "Based on recent search results, Spain won Euro 2024.")]
-        )
-
-        # Mock grounding metadata
-        mock_grounding = Mock()
-        mock_grounding.web_search_queries = ["UEFA Euro 2024 winner", "Euro 2024 final"]
-        mock_grounding.grounding_chunks = [
-            Mock(web=Mock(uri="https://uefa.com/euro2024", title="UEFA Euro 2024")),
-            Mock(web=Mock(uri="https://bbc.com/sport", title="BBC Sport")),
-        ]
-        mock_grounding.grounding_supports = [
-            Mock(
-                segment=Mock(start_index=0, end_index=50, text="Spain won Euro 2024"),
-                grounding_chunk_indices=[0, 1],
-            )
-        ]
-        mock_grounding.search_entry_point = Mock(rendered_content="<search-widget>")
-
-        response.candidates[0].grounding_metadata = mock_grounding
-        mock_client.models.generate_content = AsyncMock(return_value=response)
-
-        with patch.object(GeminiClient, "__post_init__"):
-            client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "_client", mock_client)
-
-            result = await client.get_chat_response(
-                messages=[UserMessage(content="Who won Euro 2024?")],
-                use_google_search=True,
-            )
-
-        # Verify google_search tool was added for Gemini 2.5
-        call_args = mock_client.models.generate_content.call_args[1]
-        config = call_args["config"]
-        assert hasattr(config, "tools")
-        assert len(config.tools) == 1
-        assert hasattr(config.tools[0], "google_search")
-
-        # Verify GroundingBlock is included in content
-        grounding_blocks = [
-            content for content in result.content if isinstance(content, GroundingBlock)
-        ]
-        assert len(grounding_blocks) == 1
-        grounding = grounding_blocks[0]
-
-        assert grounding.search_queries == ["UEFA Euro 2024 winner", "Euro 2024 final"]
-        assert len(grounding.grounding_chunks) == 2
-        assert grounding.grounding_chunks[0]["web"]["title"] == "UEFA Euro 2024"
-        assert len(grounding.grounding_supports) == 1
-        assert grounding.search_entry_point["rendered_content"] == "<search-widget>"
-
-    @pytest.mark.asyncio
-    async def test_google_search_enabled_gemini_15(self, mock_client):
-        """Test Google Search grounding with Gemini 1.5 models (google_search_retrieval tool)."""
-        response = create_mock_response([("text", "Recent AI developments include...")])
-
-        # Mock grounding metadata for legacy tool
-        mock_grounding = Mock()
-        mock_grounding.web_search_queries = ["AI developments 2024"]
-        mock_grounding.grounding_chunks = [
-            Mock(web=Mock(uri="https://techcrunch.com", title="TechCrunch"))
-        ]
-        mock_grounding.grounding_supports = []
-        mock_grounding.search_entry_point = None
-
-        response.candidates[0].grounding_metadata = mock_grounding
-        mock_client.models.generate_content = AsyncMock(return_value=response)
-
-        with patch.object(GeminiClient, "__post_init__"):
-            client = GeminiClient(api_key="fake_key", model="gemini-1.5-flash")
-            object.__setattr__(client, "_client", mock_client)
-
-            result = await client.get_chat_response(
-                messages=[UserMessage(content="Latest AI developments?")],
-                use_google_search=True,
-                google_search_dynamic_threshold=0.7,
-            )
-
-        # Verify google_search_retrieval tool was added for Gemini 1.5
-        call_args = mock_client.models.generate_content.call_args[1]
-        config = call_args["config"]
-        assert hasattr(config, "tools")
-        assert len(config.tools) == 1
-        assert hasattr(config.tools[0], "google_search_retrieval")
-
-        # Verify GroundingBlock is included
-        grounding_blocks = [
-            content for content in result.content if isinstance(content, GroundingBlock)
-        ]
-        assert len(grounding_blocks) == 1
-        grounding = grounding_blocks[0]
-
-        assert grounding.search_queries == ["AI developments 2024"]
-        assert len(grounding.grounding_chunks) == 1
-        assert grounding.grounding_chunks[0]["web"]["title"] == "TechCrunch"
-
-    @pytest.mark.asyncio
-    async def test_google_search_disabled(self, mock_client):
-        """Test Google Search when disabled (default behavior)."""
-        response = create_mock_response([("text", "Paris is the capital of France.")])
-        response.candidates[0].grounding_metadata = None
-        mock_client.models.generate_content = AsyncMock(return_value=response)
-
-        with patch.object(GeminiClient, "__post_init__"):
-            client = GeminiClient(api_key="fake_key")
-            object.__setattr__(client, "_client", mock_client)
-
-            result = await client.get_chat_response(
-                messages=[UserMessage(content="What is the capital of France?")],
-                # use_google_search defaults to False
-            )
-
-        # Verify no Google Search tools were added
-        call_args = mock_client.models.generate_content.call_args[1]
-        config = call_args["config"]
-        if hasattr(config, "tools") and config.tools:
-            # Should not have google_search or google_search_retrieval tools
-            for tool in config.tools:
-                assert not hasattr(tool, "google_search")
-                assert not hasattr(tool, "google_search_retrieval")
-
-        # Verify no GroundingBlock in content
-        grounding_blocks = [
-            content for content in result.content if isinstance(content, GroundingBlock)
-        ]
-        assert len(grounding_blocks) == 0
-
-    @pytest.mark.asyncio
-    async def test_google_search_with_url_context(self, mock_client):
-        """Test Google Search combined with URL context."""
-        response = create_mock_response(
-            [("text", "Analysis of the webpage and search results.")]
-        )
-
-        # Mock both grounding and URL context metadata
-        mock_grounding = Mock()
-        mock_grounding.web_search_queries = ["webpage analysis"]
-        mock_grounding.grounding_chunks = [
-            Mock(web=Mock(uri="https://example.com/analysis", title="Analysis Site"))
-        ]
-        mock_grounding.grounding_supports = []
-        mock_grounding.search_entry_point = {}
-
-        response.candidates[0].grounding_metadata = mock_grounding
-        response.candidates[0].url_context_metadata = {
-            "urls_accessed": ["https://example.com"]
-        }
-
-        mock_client.models.generate_content = AsyncMock(return_value=response)
-
-        with patch.object(GeminiClient, "__post_init__"):
-            client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "_client", mock_client)
-
-            result = await client.get_chat_response(
-                messages=[UserMessage(content="Analyze https://example.com")],
-                use_google_search=True,
-                use_url_context=True,
-            )
-
-        # Verify both tools were configured (just check that tools exist)
-        call_args = mock_client.models.generate_content.call_args[1]
-        config = call_args["config"]
-        assert hasattr(config, "tools")
-        assert len(config.tools) >= 2  # Should have at least 2 tools
-
-        # Verify both content types are present in the response
-        grounding_blocks = [
-            content for content in result.content if isinstance(content, GroundingBlock)
-        ]
-        url_context_blocks = [
-            content
-            for content in result.content
-            if isinstance(content, URLContextBlock)
-        ]
-        assert len(grounding_blocks) == 1
-        assert len(url_context_blocks) == 1
-
-    @pytest.mark.asyncio
-    async def test_google_search_streaming_config(self, mock_client):
-        """Test Google Search tool configuration in streaming mode."""
-
-        # Simple mock that avoids the iteration issues
-        async def mock_stream():
-            # Return empty stream to avoid Mock iteration issues
-            return
-            yield  # Unreachable but keeps the generator valid
-
-        mock_client.models.generate_content_stream = AsyncMock(
-            return_value=mock_stream()
-        )
-
-        with patch.object(GeminiClient, "__post_init__"):
-            client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "_client", mock_client)
-
-            try:
-                # Just start the stream to verify configuration
-                stream = client.stream_chat_response(
-                    messages=[UserMessage(content="Stream search query")],
-                    use_google_search=True,
-                )
-                # Try to get the first item (will be empty due to our mock)
-                async for _ in stream:
-                    break
-            except StopAsyncIteration:
-                # Expected since our mock stream is empty
-                pass
-
-        # Verify google_search tool was configured
-        call_args = mock_client.models.generate_content_stream.call_args[1]
-        config = call_args["config"]
-        assert hasattr(config, "tools")
-        assert len(config.tools) >= 1  # Should have at least the google_search tool
-
-    @pytest.mark.asyncio
-    async def test_google_search_no_metadata(self, mock_client):
-        """Test Google Search when no grounding metadata is returned."""
-        response = create_mock_response([("text", "Response from model knowledge.")])
-        response.candidates[0].grounding_metadata = None
-        mock_client.models.generate_content = AsyncMock(return_value=response)
-
-        with patch.object(GeminiClient, "__post_init__"):
-            client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "_client", mock_client)
-
-            result = await client.get_chat_response(
-                messages=[UserMessage(content="Simple question")],
-                use_google_search=True,
-            )
-
-        # Verify google_search tool was configured
-        call_args = mock_client.models.generate_content.call_args[1]
-        config = call_args["config"]
-        assert hasattr(config, "tools")
-        assert len(config.tools) == 1
-        assert hasattr(config.tools[0], "google_search")
-
-        # Verify no GroundingBlock when no metadata returned
-        grounding_blocks = [
-            content for content in result.content if isinstance(content, GroundingBlock)
-        ]
-        assert len(grounding_blocks) == 0
-
-    @pytest.mark.asyncio
-    async def test_google_search_structured_response(self, mock_client):
-        """Test Google Search with structured responses."""
-        from pydantic import BaseModel
-
-        class SearchResult(BaseModel):
-            answer: str
-            sources: list[str]
-
-        # Mock response for structured output
-        json_response = (
-            '{"answer": "Spain won Euro 2024", "sources": ["uefa.com", "bbc.com"]}'
-        )
-        response = create_mock_response([("text", json_response)])
-
-        # Mock grounding metadata
-        mock_grounding = Mock()
-        mock_grounding.web_search_queries = ["Euro 2024 winner"]
-        mock_grounding.grounding_chunks = [
-            Mock(web=Mock(uri="https://uefa.com", title="UEFA")),
-            Mock(web=Mock(uri="https://bbc.com", title="BBC")),
-        ]
-        mock_grounding.grounding_supports = []
-        mock_grounding.search_entry_point = {}
-
-        response.candidates[0].grounding_metadata = mock_grounding
-        mock_client.models.generate_content = AsyncMock(return_value=response)
-
-        with patch.object(GeminiClient, "__post_init__"):
-            client = GeminiClient(api_key="fake_key", model="gemini-2.5-flash")
-            object.__setattr__(client, "_client", mock_client)
-
-            result = await client.get_structured_response(
-                messages=[UserMessage(content="Who won Euro 2024?")],
-                model=SearchResult,
-                use_google_search=True,
-            )
-
-        # Verify the structured response
-        assert isinstance(result, SearchResult)
-        assert result.answer == "Spain won Euro 2024"
-        assert result.sources == ["uefa.com", "bbc.com"]
-
-        # Note: Structured responses don't return GroundingBlock in content,
-        # but the search tool should still be configured
-        call_args = mock_client.models.generate_content.call_args[1]
-        config = call_args["config"]
-        assert hasattr(config, "tools")
-        # Should have both google_search tool and response schema
-        assert hasattr(config, "response_schema")
